@@ -34,6 +34,9 @@
 /* For >= 500 for pread/pwrite and readdir_r; >= 700 for utimensat */
 #define _XOPEN_SOURCE 700
 
+/* For stat() nanosecond precision and lutimes() */
+#define _BSD_SOURCE
+
 #include <stdlib.h>
 #include <stddef.h>
 #include <stdio.h>
@@ -45,6 +48,7 @@
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
+#include <sys/time.h>
 #include <sys/statvfs.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -64,10 +68,11 @@
 #include <fuse_opt.h>
 
 #include "debug.h"
+#include "misc.h"
 #include "permchain.h"
+#include "rate_limiter.h"
 #include "userinfo.h"
 #include "usermap.h"
-#include "misc.h"
 
 /* SETTINGS */
 static struct Settings {
@@ -80,13 +85,16 @@ static struct Settings {
     const char *mntsrc;
     const char *mntdest;
     int mntsrc_fd;
-    
+
     char* original_working_dir;
     mode_t original_umask;
-    
+
     UserMap* usermap; /* From the --map option. */
     UserMap* usermap_reverse;
 
+    RateLimiter* read_limiter;
+    RateLimiter* write_limiter;
+
     enum CreatePolicy {
         CREATE_AS_USER,
         CREATE_AS_MOUNTER
@@ -111,7 +119,7 @@ static struct Settings {
         CHMOD_IGNORE,
         CHMOD_DENY
     } chmod_policy;
-    
+
     int chmod_allow_x;
 
     struct permchain *chmod_permchain; /* the --chmod-filter option */
@@ -132,7 +140,7 @@ static struct Settings {
 
     int ctime_from_mtime;
     int hide_hard_links;
-    
+
 } settings;
 
 
@@ -248,13 +256,19 @@ static int getattr_common(const char *procpath, struct stat *stbuf)
     /* Copy mtime (file content modification time)
        to ctime (inode/status change time)
        if the user asked for that */
-    if (settings.ctime_from_mtime)
+    if (settings.ctime_from_mtime) {
+#ifdef HAVE_STAT_NANOSEC
+        // TODO: does this work on OS X?
+        stbuf->st_ctim = stbuf->st_mtim;
+#else
         stbuf->st_ctime = stbuf->st_mtime;
+#endif
+    }
 
     /* Possibly map user/group */
     stbuf->st_uid = usermap_get_uid_or_default(settings.usermap, stbuf->st_uid, stbuf->st_uid);
     stbuf->st_gid = usermap_get_gid_or_default(settings.usermap, stbuf->st_gid, stbuf->st_gid);
-    
+
     /* Report user-defined owner/group if specified */
     if (settings.new_uid != -1)
         stbuf->st_uid = settings.new_uid;
@@ -316,7 +330,7 @@ static void chown_new_file(const char *path, struct fuse_context *fc, int (*chow
         file_owner = -1;
         file_group = -1;
     }
-    
+
     file_owner = usermap_get_uid_or_default(settings.usermap_reverse, fc->uid, file_owner);
     file_group = usermap_get_gid_or_default(settings.usermap_reverse, fc->gid, file_group);
 
@@ -338,7 +352,7 @@ static void *bindfs_init()
 {
     assert(settings.permchain != NULL);
     assert(settings.mntsrc_fd > 0);
-    
+
     maybe_stdout_stderr_to_file();
 
     if (fchdir(settings.mntsrc_fd) != 0) {
@@ -423,9 +437,9 @@ static int bindfs_readdir(const char *path, void *buf, fuse_fill_dir_t filler,
     struct stat st;
     int result = 0;
     long pc_ret;
-    
+
     path = process_path(path);
-    
+
     pc_ret = pathconf(path, _PC_NAME_MAX);
     if (pc_ret < 0) {
         DPRINTF("pathconf failed: %s (%d)", strerror(errno), errno);
@@ -443,7 +457,7 @@ static int bindfs_readdir(const char *path, void *buf, fuse_fill_dir_t filler,
         if (de == NULL) {
             break;
         }
-        
+
         memset(&st, 0, sizeof(st));
         st.st_ino = de->d_ino;
         st.st_mode = de->d_type << 12;
@@ -640,7 +654,7 @@ static int bindfs_chown(const char *path, uid_t uid, gid_t gid)
             return -EPERM;
         }
     }
-    
+
     if (gid != -1) {
         switch (settings.chgrp_policy) {
         case CHGRP_NORMAL:
@@ -690,13 +704,25 @@ static int bindfs_ftruncate(const char *path, off_t size,
     return 0;
 }
 
-static int bindfs_utimens(const char *path, const struct timespec tv[2])
+static int bindfs_utimens(const char *path, const struct timespec ts[2])
 {
     int res;
 
     path = process_path(path);
 
-    res = utimensat(settings.mntsrc_fd, path, tv, AT_SYMLINK_NOFOLLOW);
+#ifdef HAVE_UTIMENSAT
+    res = utimensat(settings.mntsrc_fd, path, ts, AT_SYMLINK_NOFOLLOW);
+#elif HAVE_LUTIMES
+    struct timeval tv[2];
+    tv[0].tv_sec = ts[0].tv_sec;
+    tv[0].tv_usec = ts[0].tv_nsec / 1000;
+    tv[1].tv_sec = ts[1].tv_sec;
+    tv[1].tv_usec = ts[1].tv_nsec / 1000;
+    res = lutimes(path, tv);
+#else
+#error "No symlink-compatible utime* function available."
+#endif
+
     if (res == -1)
         return -errno;
 
@@ -743,7 +769,11 @@ static int bindfs_read(const char *path, char *buf, size_t size, off_t offset,
 {
     int res;
     (void) path;
-    
+
+    if (settings.read_limiter) {
+        rate_limiter_wait(settings.read_limiter, size);
+    }
+
     res = pread(fi->fh, buf, size, offset);
     if (res == -1)
         res = -errno;
@@ -757,6 +787,10 @@ static int bindfs_write(const char *path, const char *buf, size_t size,
     int res;
     (void) path;
 
+    if (settings.write_limiter) {
+        rate_limiter_wait(settings.write_limiter, size);
+    }
+
     res = pwrite(fi->fh, buf, size, offset);
     if (res == -1)
         res = -errno;
@@ -780,7 +814,7 @@ static int bindfs_statfs(const char *path, struct statvfs *stbuf)
 static int bindfs_release(const char *path, struct fuse_file_info *fi)
 {
     (void) path;
-    
+
     close(fi->fh);
 
     return 0;
@@ -813,7 +847,7 @@ static int bindfs_setxattr(const char *path, const char *name, const char *value
                            size_t size, int flags)
 {
     DPRINTF("setxattr %s %s=%s", path, name, value);
-    
+
     if (settings.xattr_policy == XATTR_READ_ONLY)
         return -EACCES;
 
@@ -834,7 +868,7 @@ static int bindfs_getxattr(const char *path, const char *name, char *value,
     int res;
 
     DPRINTF("getxattr %s %s", path, name);
-    
+
     path = process_path(path);
     /* fuse checks permissions for us */
 #ifdef HAVE_LGETXATTR
@@ -852,7 +886,7 @@ static int bindfs_listxattr(const char *path, char *list, size_t size)
     int res;
 
     DPRINTF("listxattr %s", path);
-    
+
     path = process_path(path);
     /* fuse checks permissions for us */
 #ifdef HAVE_LLISTXATTR
@@ -934,24 +968,24 @@ static void print_usage(const char *progname)
            "  -V      --version         Print version number and exit.\n"
            "\n"
            "File ownership:\n"
-           "  -u      --force-user      Set file owner.\n"
-           "  -g      --force-group     Set file group.\n"
-           "  -m      --mirror          Comma-separated list of users who will see\n"
+           "  -u      --force-user=...  Set file owner.\n"
+           "  -g      --force-group=... Set file group.\n"
+           "  -m      --mirror=...      Comma-separated list of users who will see\n"
            "                            themselves as the owners of all files.\n"
-           "  -M      --mirror-only     Like --mirror but disallow access for\n"
+           "  -M      --mirror-only=... Like --mirror but disallow access for\n"
            "                            all other users.\n"
            " --map=user1/user2:...      Let user2 see files of user1 as his own.\n"
            "\n"
            "Permission bits:\n"
-           "  -p      --perms           Specify permissions, similar to chmod\n"
+           "  -p      --perms=...       Specify permissions, similar to chmod\n"
            "                            e.g. og-x,og+rD,u=rwX,g+rw  or  0644,a+X\n"
            "\n"
            "File creation policy:\n"
            "  --create-as-user          New files owned by creator (default for root). *\n"
            "  --create-as-mounter       New files owned by fs mounter (default for users).\n"
-           "  --create-for-user         New files owned by specified user. *\n"
-           "  --create-for-group        New files owned by specified group. *\n"
-           "  --create-with-perms       Alter permissions of new files.\n"
+           "  --create-for-user=...     New files owned by specified user. *\n"
+           "  --create-for-group=...    New files owned by specified group. *\n"
+           "  --create-with-perms=...   Alter permissions of new files.\n"
            "\n"
            "Chown policy:\n"
            "  --chown-normal            Try to chown the original files (the default).\n"
@@ -967,7 +1001,7 @@ static void print_usage(const char *progname)
            "  --chmod-normal            Try to chmod the original files (the default).\n"
            "  --chmod-ignore            Have all chmods fail silently.\n"
            "  --chmod-deny              Have all chmods fail with 'permission denied'.\n"
-           "  --chmod-filter            Change permissions of chmod requests.\n"
+           "  --chmod-filter=...        Change permissions of chmod requests.\n"
            "  --chmod-allow-x           Allow changing file execute bits in any case.\n"
            "\n"
            "Extended attribute policy:\n"
@@ -975,6 +1009,10 @@ static void print_usage(const char *progname)
            "  --xattr-ro                Read-only xattr operations.\n"
            "  --xattr-rw                Read-write xattr operations (the default).\n"
            "\n"
+           "Rate limits:\n"
+           "  --read-rate=...           Limit to bytes/sec that can be read.\n"
+           "  --write-rate=...          Limit to bytes/sec that can be written.\n"
+           "\n"
            "Miscellaneous:\n"
            "  -n      --no-allow-other  Do not add -o allow_other to fuse options.\n"
            "  --realistic-permissions   Hide permission bits for actions mounter can't do.\n"
@@ -1165,7 +1203,7 @@ static int parse_mirrored_users(char* mirror)
             assert(j == settings.num_mirrored_members);
         }
     }
-    
+
     return 1;
 }
 
@@ -1177,11 +1215,11 @@ static int parse_user_map(UserMap *map, UserMap *reverse_map, char *spec)
     uid_t uid_from, uid_to;
     gid_t gid_from, gid_to;
     UsermapStatus status;
-    
+
     while (*p != '\0') {
         free(tmpstr);
         tmpstr = strdup_until(p, ",:");
-        
+
         if (tmpstr[0] == '@') { /* group */
             q = strstr(tmpstr, "/@");
             if (!q) {
@@ -1198,7 +1236,7 @@ static int parse_user_map(UserMap *map, UserMap *reverse_map, char *spec)
                 fprintf(stderr, "Invalid group: %s\n", tmpstr);
                 goto fail;
             }
-            
+
             status = usermap_add_gid(map, gid_from, gid_to);
             if (status != 0) {
                 fprintf(stderr, "%s\n", usermap_errorstr(status));
@@ -1209,9 +1247,9 @@ static int parse_user_map(UserMap *map, UserMap *reverse_map, char *spec)
                 fprintf(stderr, "%s\n", usermap_errorstr(status));
                 goto fail;
             }
-            
+
         } else {
-            
+
             q = strstr(tmpstr, "/");
             if (!q) {
                 fprintf(stderr, "Invalid syntax: expected user1/user2 but got `%s`\n", tmpstr);
@@ -1227,7 +1265,7 @@ static int parse_user_map(UserMap *map, UserMap *reverse_map, char *spec)
                 fprintf(stderr, "Invalid username: %s\n", tmpstr);
                 goto fail;
             }
-            
+
             status = usermap_add_uid(map, uid_from, uid_to);
             if (status != 0) {
                 fprintf(stderr, "%s\n", usermap_errorstr(status));
@@ -1239,7 +1277,7 @@ static int parse_user_map(UserMap *map, UserMap *reverse_map, char *spec)
                 goto fail;
             }
         }
-        
+
         while (*p != '\0' && *p != ',' && *p != ':') {
             ++p;
         }
@@ -1247,10 +1285,10 @@ static int parse_user_map(UserMap *map, UserMap *reverse_map, char *spec)
             ++p;
         }
     }
-    
+
     free(tmpstr);
     return 1;
-    
+
 fail:
     free(tmpstr);
     return 0;
@@ -1261,16 +1299,16 @@ static void maybe_stdout_stderr_to_file()
     /* TODO: make this a command line option. */
 #if 0
     int fd;
-    
+
     const char *filename = "bindfs.log";
     char *path = malloc(strlen(settings.original_working_dir) + 1 + strlen(filename) + 1);
     strcpy(path, settings.original_working_dir);
     strcat(path, "/");
     strcat(path, filename);
-    
+
     fd = open(path, O_CREAT | O_WRONLY, 0666);
     free(path);
-    
+
     fchmod(fd, 0777 & ~settings.original_umask);
     fflush(stdout);
     fflush(stderr);
@@ -1296,7 +1334,7 @@ static void setup_signal_handling()
     sa.sa_handler = signal_handler;
     sigemptyset(&sa.sa_mask);
     sa.sa_flags = 0;
-    
+
     sigaction(SIGUSR1, &sa, NULL);
 }
 
@@ -1309,6 +1347,16 @@ static void atexit_func()
 {
     free(settings.original_working_dir);
     settings.original_working_dir = NULL;
+    if (settings.read_limiter) {
+        rate_limiter_destroy(settings.read_limiter);
+        free(settings.read_limiter);
+        settings.read_limiter = NULL;
+    }
+    if (settings.write_limiter) {
+        rate_limiter_destroy(settings.write_limiter);
+        free(settings.write_limiter);
+        settings.write_limiter = NULL;
+    }
     usermap_destroy(settings.usermap);
     settings.usermap = NULL;
     usermap_destroy(settings.usermap_reverse);
@@ -1339,6 +1387,8 @@ int main(int argc, char *argv[])
         char *mirror;
         char *mirror_only;
         char *map;
+        char *read_rate;
+        char *write_rate;
         char *create_for_user;
         char *create_for_group;
         char *create_with_perms;
@@ -1360,43 +1410,46 @@ int main(int argc, char *argv[])
     static const struct fuse_opt options[] = {
         OPT2("-h", "--help", OPTKEY_HELP),
         OPT2("-V", "--version", OPTKEY_VERSION),
-        
+
         OPT_OFFSET3("-u %s", "--force-user=%s", "force-user=%s", user, -1),
         OPT_OFFSET3("-g %s", "--force-group=%s", "force-group=%s", group, -1),
-        
+
         OPT_OFFSET3("--user=%s", "--owner=%s", "owner=%s", deprecated_user, -1),
         OPT_OFFSET2("--group=%s", "group=%s", deprecated_group, -1),
-        
+
         OPT_OFFSET3("-p %s", "--perms=%s", "perms=%s", perms, -1),
         OPT_OFFSET3("-m %s", "--mirror=%s", "mirror=%s", mirror, -1),
         OPT_OFFSET3("-M %s", "--mirror-only=%s", "mirror-only=%s", mirror_only, -1),
         OPT_OFFSET2("--map=%s", "map=%s", map, -1),
         OPT_OFFSET3("-n", "--no-allow-other", "no-allow-other", no_allow_other, -1),
-        
+
+        OPT_OFFSET2("--read-rate=%s", "read-rate=%s", read_rate, -1),
+        OPT_OFFSET2("--write-rate=%s", "write-rate=%s", write_rate, -1),
+
         OPT2("--create-as-user", "create-as-user", OPTKEY_CREATE_AS_USER),
         OPT2("--create-as-mounter", "create-as-mounter", OPTKEY_CREATE_AS_MOUNTER),
         OPT_OFFSET2("--create-for-user=%s", "create-for-user=%s", create_for_user, -1),
         OPT_OFFSET2("--create-for-group=%s", "create-for-group=%s", create_for_group, -1),
         OPT_OFFSET2("--create-with-perms=%s", "create-with-perms=%s", create_with_perms, -1),
-        
+
         OPT2("--chown-normal", "chown-normal", OPTKEY_CHOWN_NORMAL),
         OPT2("--chown-ignore", "chown-ignore", OPTKEY_CHOWN_IGNORE),
         OPT2("--chown-deny", "chown-deny", OPTKEY_CHOWN_DENY),
-        
+
         OPT2("--chgrp-normal", "chgrp-normal", OPTKEY_CHGRP_NORMAL),
         OPT2("--chgrp-ignore", "chgrp-ignore", OPTKEY_CHGRP_IGNORE),
         OPT2("--chgrp-deny", "chgrp-deny", OPTKEY_CHGRP_DENY),
-        
+
         OPT2("--chmod-normal", "chmod-normal", OPTKEY_CHMOD_NORMAL),
         OPT2("--chmod-ignore", "chmod-ignore", OPTKEY_CHMOD_IGNORE),
         OPT2("--chmod-deny", "chmod-deny", OPTKEY_CHMOD_DENY),
         OPT_OFFSET2("--chmod-filter=%s", "chmod-filter=%s", chmod_filter, -1),
         OPT2("--chmod-allow-x", "chmod-allow-x", OPTKEY_CHMOD_ALLOW_X),
-        
+
         OPT2("--xattr-none", "xattr-none", OPTKEY_XATTR_NONE),
         OPT2("--xattr-ro", "xattr-ro", OPTKEY_XATTR_READ_ONLY),
         OPT2("--xattr-rw", "xattr-rw", OPTKEY_XATTR_READ_WRITE),
-        
+
         OPT2("--realistic-permissions", "realistic-permissions", OPTKEY_REALISTIC_PERMISSIONS),
         OPT2("--ctime-from-mtime", "ctime-from-mtime", OPTKEY_CTIME_FROM_MTIME),
         OPT2("--hide-hard-links", "hide-hard-links", OPTKEY_HIDE_HARD_LINKS),
@@ -1413,6 +1466,8 @@ int main(int argc, char *argv[])
     settings.permchain = permchain_create();
     settings.usermap = usermap_create();
     settings.usermap_reverse = usermap_create();
+    settings.read_limiter = NULL;
+    settings.write_limiter = NULL;
     settings.new_uid = -1;
     settings.new_gid = -1;
     settings.create_for_uid = -1;
@@ -1437,7 +1492,7 @@ int main(int argc, char *argv[])
     settings.ctime_from_mtime = 0;
     settings.hide_hard_links = 0;
     atexit(&atexit_func);
-    
+
     /* Parse options */
     if (fuse_opt_parse(&args, &od, options, &process_option) == -1)
         return 1;
@@ -1447,7 +1502,7 @@ int main(int argc, char *argv[])
         print_usage(my_basename(argv[0]));
         return 1;
     }
-    
+
     /* Check for deprecated options */
     if (od.deprecated_user) {
         fprintf(stderr, "Deprecation warning: please use --force-user instead of --user or --owner.\n");
@@ -1477,7 +1532,29 @@ int main(int argc, char *argv[])
             return 1;
         }
     }
-    
+
+    /* Parse rate limits */
+    if (od.read_rate) {
+        double rate;
+        if (parse_byte_count(od.read_rate, &rate) && rate > 0) {
+            settings.read_limiter = malloc(sizeof(RateLimiter));
+            rate_limiter_init(settings.read_limiter, rate, &gettimeofday_clock);
+        } else {
+            fprintf(stderr, "Error: Invalid --read-rate.\n");
+            return 1;
+        }
+    }
+    if (od.write_rate) {
+        double rate;
+        if (parse_byte_count(od.write_rate, &rate) && rate > 0) {
+            settings.write_limiter = malloc(sizeof(RateLimiter));
+            rate_limiter_init(settings.write_limiter, rate, &gettimeofday_clock);
+        } else {
+            fprintf(stderr, "Error: Invalid --write-rate.\n");
+            return 1;
+        }
+    }
+
     /* Parse usermap */
     if (od.map) {
         if (getuid() != 0) {
@@ -1560,11 +1637,11 @@ int main(int argc, char *argv[])
 
     /* We want the kernel to do our access checks for us based on what getattr gives it. */
     fuse_opt_add_arg(&args, "-odefault_permissions");
-    
+
     /* We want to mirror inodes. */
     fuse_opt_add_arg(&args, "-ouse_ino");
     fuse_opt_add_arg(&args, "-oreaddir_ino");
-    
+
     /* We need to disable the attribute cache whenever two users
        can see different attributes. For now, only mirroring can do that. */
     if (is_mirroring_enabled()) {
